/*************************************************************
*  correct_center.c  Correct Center Position  Ver.1.0        *
*                                          Date:14 May 2004  *
*                                Programed by Hiroki.Segawa  *
*************************************************************/

/*****************************************************************************
*       This Source file contains these function.                            *
*                                                                            *
* int correct_center(void);      Calculation of dtheta,dx,dy,move parameter  *
* int primary_pos(int,int,int);  move method to correct Center position      *
*****************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <pthread.h>
#include "func_extern.h"
#include "control.h"
#include "irs_ts_ws.h"
#include "pwm_enc.h"
#include "func_extern.h"
#include "uss.h"

int primary_pos_count;// count of this function call

int correct_center(void){
	float r1_P, r2_P, theta1_P, theta2_P; // Primary Search
	float r1_S, r2_S, theta1_S, theta2_S; // Secondary Search
	float ans_b, ans_a; // answer of equation
	float dtheta,dx,dy; // final answer
	float tmp1, tmp2, tmp3, tmp4; // variables for calculation
	float mark = 1.0; // plus or minus
	float alpha, beta; // variables for moving to primary position
	float ans_tmp[2], dx_tmp[2], dy_tmp[2]; // analysis of answer
	int deg1, deg2, distance; // variables for pwm functions
	int i,flag;
	primary_pos_count=0;
	
	// Input Parameter
	r1_P = post.dis[0]; // [mm]
	r2_P = post.dis[1]; // [mm]
	theta1_P = (post.ang[0]+90)*pi/180; // [rad]
	theta2_P = (post.ang[1]+90)*pi/180; // [rad]
	r1_S = post_tmp.dis[0]; // [mm]
	r2_S = post_tmp.dis[1]; // [mm]
	theta1_S = (post_tmp.ang[0]+90)*pi/180; // [rad]
	theta2_S = (post_tmp.ang[1]+90)*pi/180; // [rad]

	// Calculation of dtheta
	tmp1 = 1 / (2 * r1_P * r1_S - 2 * r2_P * r2_S);
	tmp2 = 2 * r1_P * r1_S * (theta1_S - theta1_P) - 2 * r2_P * r2_S * (theta2_S - theta2_P);
	tmp3 = pow((r1_P - r1_S),2) - pow((r2_P - r2_S),2) + r1_P * r1_S * pow((theta1_S - theta1_P),2) - r2_P * r2_S * pow((theta2_S - theta2_P),2);
	tmp4 = pow(tmp2,2) - 4 * (r1_P * r1_S - r2_P * r2_S) * tmp3;
	if(tmp4<0){
		tmp4 = -tmp4;
		mark = -1.0; // minus flag
	}
	ans_b = tmp1 * tmp2;
	ans_a = tmp1 * sqrt(tmp4);
	ans_a *= mark;  // because,forbid imaginary number
	// got two answers(dtheta)
	ans_tmp[0] = ans_b + ans_a;
	ans_tmp[1] = ans_b - ans_a;
	// got two answers(dx,dy)
	for(i=0; i<2; i++){
		dtheta = -ans_tmp[i];
		dx_tmp[i] = r1_P * cos(theta1_P) - r1_S * cos(theta1_S + dtheta); 
		dy_tmp[i] = r1_P * sin(theta1_P) - r1_S * sin(theta1_S + dtheta);
		printf("dtheta[%d]:%.2f  dx_t[%d]:%.2f  dy_t[%d]:%.2f\n",i,dtheta,i,dx_tmp[i],i,dy_tmp[i]);
	}
	// analysis of answer
	if(fabs(dx_tmp[0])<=fabs(dx_tmp[1]) && fabs(dy_tmp[0])<=fabs(dy_tmp[1])){
		dtheta = -ans_tmp[0];
		dx = dx_tmp[0];
		dy = dy_tmp[0];
	}
	else if(fabs(dx_tmp[0])>=fabs(dx_tmp[1]) && fabs(dy_tmp[0])>=fabs(dy_tmp[1])){
		dtheta = -ans_tmp[1];
		dx = dx_tmp[1];
		dy = dy_tmp[1];
	}
	else{
		if(fabs(fabs(dx_tmp[0])-fabs(dy_tmp[0]))<fabs(fabs(dx_tmp[1])-fabs(dy_tmp[1]))){
			dtheta = -ans_tmp[0];
			dx = dx_tmp[0];
			dy = dy_tmp[0];
		}
		else{
			dtheta = -ans_tmp[1];
			dx = dx_tmp[1];
			dy = dy_tmp[1];
		}
			
	}
	// got Parameter to move to Primary Position
	alpha = atan(fabs(dy/dx)); // [rad]
	distance = (int)(sqrt(pow(dx,2) + pow(dy,2))); // move distance [mm]
	deg1 = 0; // initialize of first rotate angle [deg]
	deg2 = (int)(((pi/2) - alpha) * 180 / pi); // second rotate angle [deg]
	flag=0; // move action depend on quadrand
	// parameter of first quadrand
	if(dx>=0 && dy>=0){
		beta = (pi/2) - alpha + dtheta; // [rad]
		deg1 = (int)(beta * 180 / pi); // [deg]
		flag=1;
	}
	// parameter of second quadrand
	else if(dx<0 && dy>0){
		beta = (pi/2) - alpha - dtheta;
		deg1 = (int)(beta * 180 / pi);
		primary_pos(deg1,-distance,-deg2) ;
		flag=2;
	}
	// parameter of third quadrand
	else if(dx<0 && dy<0){
		beta = (pi/2) - alpha + dtheta; // [rad]
		deg1 = (int)(beta * 180 / pi); // [deg]
		flag=3;
	}
	// parameter of fourth quadrand
	else if(dx>0 && dy<0){
		beta = (pi/2) - alpha - dtheta;
		deg1 = (int)(beta * 180 / pi);
		flag=4;
	}
	// Debug Message
	printf("\n=============Debug Message=============\n");
	printf("ans_b + ans_a = %.3f [deg]\n",-(ans_b+ans_a)*180/pi);
	printf("ans_b - ans_a = %.3f [deg]\n",-(ans_b-ans_a)*180/pi);
	printf("dtheta     :%.2f [deg]\n",dtheta*180/pi);
	printf("dx         :%.2f [mm]\n",dx);
	printf("dy         :%.2f [mm]\n",dy);
	printf("alpha      :%3d [deg]\n",(int)(alpha*180/pi));
	printf("beta(deg1) :%3d [deg]\n",deg1);
	printf("deg2       :%3d [deg]\n",deg2);
	printf("distance   :%3d [mm]\n",distance);
	printf("==============End Message==============\n");
	switch(flag){
		case 1:
			primary_pos(-deg1,-distance,deg2);
			break;
		case 2:
			primary_pos(deg1,-distance,-deg2);
			break;	
		case 3:
			primary_pos(-deg1,distance,deg2);
			break;	
		case 4:
			primary_pos(deg1,distance,-deg2);
			break;	
		default:
			break;
	}
	return 0;
}
int primary_pos(int first_rotate, int distance, int second_rotate){
	int time;
	primary_pos_count++;
	if(primary_pos_count>1) return -1; // sometimes,this function call 2 times.
	usleep(1000000);
	// first rotate
	time = first_rotate * 20;
	lqr_rotate(first_rotate, time);
	usleep(1000000);
	// straight
	time = distance * 5;
	lqr_straight(distance, time);
	usleep(1000000);
	// second rotate
	time = second_rotate * 20;
	lqr_rotate(second_rotate, time);
	usleep(1000000);
	return 0;
}
