/*************************************************************
*     pwm_enc.h  Tire Parameter & Structure  Ver.1.0         *
*                                          Date:14 May 2004  *
*                                Programed by Hiroki.Segawa  *
*************************************************************/

/* For the task control */
#define COMMAND_FIFO  5
#define START_TASK    1
#define STOP_TASK     2
#define RESTART_TASK  3
#define TASK_CONTROL_FIFO_OFFSET  4
#define COUNTER_RESET  1

/* For Rotary Encoder */
/* Data of tire & motor */
//#define diameter   63.0    /* diameter of the tire [mm] */
#define diameter   67.3    /* diameter of the tire [mm] */
#define dist_ct	   112.5   /* distance between mirs-center and tire [mm] */
//#define dist_ct	   110.0   /* distance between mirs-center and tire [mm] */
#define pi         3.141592  /* pi */
//#define peri       20      /* execution period of RT-Task [msec] */
#define gear       16      /* gear ratio */
#define cpr        100     /* the number of counts a revolution */
//#define intarval   234.0   /* the intarval between the tires */
#define intarval   (dist_ct*2)   /* the intarval between the tires */
#define adj        1.0

/* For PWM */
#define KEEP_SPEED -999
#define DUTY_MODE   123
#define PID_MODE    456
#define LQR_MODE    789

/* define the I/O ports */
#define PWM_LEFT_WRITE   0x0110
#define PWM_RIGHT_WRITE  0x0112
#define LEFT_COUNTER     0x0120
#define RIGHT_COUNTER    0x0122

struct my_msg_struct{
	int command;     /* command to control RT-task */
	int task;        /* number of the RT-task */
	int period;      /* execution period of the RT-task */
};

struct count_data{
	int left_count;   /* pulses of left counter */
	int right_count;  /* pulses of right counter */
	int left_sum;     /* sum of puslses of left counter */
	int right_sum;    /* sum of puslses of right counter */
	volatile int br;
};

struct send_data{
    double athetar;
    double athetal;
    double thetar;
    double thetal;
    double aomegar;
    double aomegal;
    double omegar;
    double omegal;
	double tn;
	int br;
};

struct pwm_speed_data{
	int l_speed;     /* velocity of left wheel */
	int r_speed;     /* velocity of right wheel */
	int tup;         /* target time of control */
	int mode;        /* Control mode(Duty,Stop,Lqr) */
};
